@@ -6,9 +6,8 @@ import webbrowser
 from collections import defaultdict
 
 import numpy as np
-from docarray import Document
 
-from jina.helper import colored
+from docarray import Document
 from jina.logging.predefined import default_logger
 from jina.logging.profile import ProgressBar
 
@@ -164,11 +163,8 @@ def write_html(html_path):
             f'if not you may open {url_html_path} manually'
         )
 
-    colored_url = colored(
-        'https://github.com/jina-ai/jina', color='cyan', attrs='underline'
-    )
     default_logger.info(
-        f'🤩 Intrigued? Play with `jina hello fashion --help` and learn more about Jina at {colored_url}'
+        f'🤩 Intrigued? Play with `jina hello fashion --help` and [link=https://github.com/jina-ai/jina]learn more about Jina[/]'
     )
 
 
@@ -187,13 +183,16 @@ def download_data(targets, download_proxy=None, task_name='download fashion-mnis
             {'http': download_proxy, 'https': download_proxy}
         )
         opener.add_handler(proxy)
+
     urllib.request.install_opener(opener)
-    with ProgressBar(description=task_name) as t:
+    with ProgressBar(total_length=len(targets), description=task_name) as t:
         for k, v in targets.items():
             if not os.path.exists(v['filename']):
                 urllib.request.urlretrieve(
-                    v['url'], v['filename'], reporthook=lambda *x: t.update(0.01)
+                    v['url'],
+                    v['filename'],
                 )
+                t.update()
             if k == 'index-labels' or k == 'query-labels':
                 v['data'] = load_labels(v['filename'])
             if k == 'index' or k == 'query':
