@@ -2,7 +2,6 @@ import asyncio
 import contextlib
 import ipaddress
 import os
-import time
 from collections import defaultdict
 from typing import Dict, List, Optional, Tuple
 from urllib.parse import urlparse
@@ -15,7 +14,7 @@ from grpc_reflection.v1alpha.reflection_pb2_grpc import ServerReflectionStub
 from jina.enums import PollingType
 from jina.importer import ImportExtensions
 from jina.logging.logger import JinaLogger
-from jina.proto import jina_pb2_grpc
+from jina.proto import jina_pb2, jina_pb2_grpc
 from jina.types.request import Request
 from jina.types.request.control import ControlRequest
 from jina.types.request.data import DataRequest
@@ -37,7 +36,7 @@ class ReplicaList:
         self._connections = []
         self._address_to_connection_idx = {}
         self._address_to_channel = {}
-        self._rr_counter = 0
+        self._rr_counter = 0  # round robin counter
         self.summary = summary
 
     def add_connection(self, address: str):
@@ -157,6 +156,7 @@ class GrpcConnectionPool:
             'jina.JinaControlRequestRPC': jina_pb2_grpc.JinaControlRequestRPCStub,
             'jina.JinaDataRequestRPC': jina_pb2_grpc.JinaDataRequestRPCStub,
             'jina.JinaSingleDataRequestRPC': jina_pb2_grpc.JinaSingleDataRequestRPCStub,
+            'jina.JinaDiscoverEndpointsRPC': jina_pb2_grpc.JinaDiscoverEndpointsRPCStub,
             'jina.JinaRPC': jina_pb2_grpc.JinaRPCStub,
         }
 
@@ -179,8 +179,33 @@ class GrpcConnectionPool:
             self.data_list_stub = stubs['jina.JinaDataRequestRPC']
             self.single_data_stub = stubs['jina.JinaSingleDataRequestRPC']
             self.stream_stub = stubs['jina.JinaRPC']
+            self.endpoints_discovery_stub = stubs['jina.JinaDiscoverEndpointsRPC']
             self._initialized = True
 
+        async def send_discover_endpoint(
+            self,
+            timeout: Optional[float] = None,
+        ) -> Tuple:
+            """
+            Use the endpoint discovery stub to request for the Endpoints Exposed by an Executor
+
+            :param timeout: defines timeout for sending request
+
+            :returns: Tuple of response and metadata about the response
+            """
+            if not self._initialized:
+                await self._init_stubs()
+
+            call_result = self.endpoints_discovery_stub.endpoint_discovery(
+                jina_pb2.google_dot_protobuf_dot_empty__pb2.Empty(),
+                timeout=timeout,
+            )
+            metadata, response = (
+                await call_result.trailing_metadata(),
+                await call_result,
+            )
+            return response, metadata
+
         async def send_requests(
             self,
             requests: List[Request],
@@ -279,13 +304,19 @@ class GrpcConnectionPool:
             self._add_connection(deployment, head_id, address, 'heads')
 
         def get_replicas(
-            self, deployment: str, head: bool, entity_id: Optional[int] = None
+            self,
+            deployment: str,
+            head: bool,
+            entity_id: Optional[int] = None,
+            increase_access_count: bool = True,
         ) -> ReplicaList:
             if deployment in self._deployments:
                 type_ = 'heads' if head else 'shards'
                 if entity_id is None and head:
                     entity_id = 0
-                return self._get_connection_list(deployment, type_, entity_id)
+                return self._get_connection_list(
+                    deployment, type_, entity_id, increase_access_count
+                )
             else:
                 self._logger.debug(
                     f'Unknown deployment {deployment}, no replicas available'
@@ -312,12 +343,17 @@ class GrpcConnectionPool:
             self._deployments.clear()
 
         def _get_connection_list(
-            self, deployment: str, type_: str, entity_id: Optional[int] = None
+            self,
+            deployment: str,
+            type_: str,
+            entity_id: Optional[int] = None,
+            increase_access_count: bool = True,
         ) -> ReplicaList:
             try:
                 if entity_id is None and len(self._deployments[deployment][type_]) > 0:
                     # select a random entity
-                    self._access_count[deployment] += 1
+                    if increase_access_count:
+                        self._access_count[deployment] += 1
                     return self._deployments[deployment][type_][
                         self._access_count[deployment]
                         % len(self._deployments[deployment][type_])
@@ -332,9 +368,11 @@ class GrpcConnectionPool:
                 ):
                     # This can happen as a race condition when removing connections while accessing it
                     # In this case we don't care for the concrete entity, so retry with the first one
-                    return self._get_connection_list(deployment, type_, 0)
+                    return self._get_connection_list(
+                        deployment, type_, 0, increase_access_count
+                    )
                 self._logger.debug(
-                    f'Did not find a connection for deployment {deployment}, type {type} and entity_id {entity_id}. There are {len(self._deployments[deployment][type]) if deployment in self._deployments else 0} available connections for this deployment and type. '
+                    f'did not find a connection for deployment {deployment}, type {type} and entity_id {entity_id}. There are {len(self._deployments[deployment][type]) if deployment in self._deployments else 0} available connections for this deployment and type. '
                 )
                 return None
 
@@ -359,12 +397,12 @@ class GrpcConnectionPool:
                 address
             ):
                 self._logger.debug(
-                    f'Adding connection for deployment {deployment}/{type}/{entity_id} to {address}'
+                    f'adding connection for deployment {deployment}/{type}/{entity_id} to {address}'
                 )
                 self._deployments[deployment][type][entity_id].add_connection(address)
             else:
                 self._logger.debug(
-                    f'Ignoring activation of pod, {address} already known'
+                    f'ignoring activation of pod, {address} already known'
                 )
 
         async def remove_head(self, deployment, address, head_id: Optional[int] = 0):
@@ -383,7 +421,7 @@ class GrpcConnectionPool:
                 and entity_id in self._deployments[deployment][type]
             ):
                 self._logger.debug(
-                    f'Removing connection for deployment {deployment}/{type}/{entity_id} to {address}'
+                    f'removing connection for deployment {deployment}/{type}/{entity_id} to {address}'
                 )
                 connection = await self._deployments[deployment][type][
                     entity_id
@@ -433,6 +471,7 @@ class GrpcConnectionPool:
         else:
             self._summary_time = contextlib.nullcontext()
         self._connections = self._ConnectionPoolMap(self._logger, self._summary_time)
+        self._deployment_address_map = {}
 
     def send_request(
         self,
@@ -504,6 +543,29 @@ class GrpcConnectionPool:
 
         return results
 
+    def send_discover_endpoint(
+        self,
+        deployment: str,
+        head: bool = True,
+        shard_id: Optional[int] = None,
+        timeout: Optional[float] = None,
+    ) -> asyncio.Task:
+        """Sends a discover Endpoint call to target.
+
+        :param deployment: name of the Jina deployment to send the request to
+        :param head: If True it is send to the head, otherwise to the worker pods
+        :param shard_id: Send to a specific shard of the deployment, ignored for polling ALL
+        :param timeout: timeout for sending the requests
+        :return: asyncio.Task items to send call
+        """
+        connection = None
+        connection_list = self._connections.get_replicas(
+            deployment, head, shard_id, False
+        )
+        if connection_list:
+            connection = connection_list.get_next_connection()
+        return self._send_discover_endpoint(connection, timeout=timeout)
+
     def send_request_once(
         self,
         request: Request,
@@ -553,7 +615,7 @@ class GrpcConnectionPool:
             return self._send_requests(requests, connection, endpoint, timeout=timeout)
         else:
             self._logger.debug(
-                f'No available connections for deployment {deployment} and shard {shard_id}'
+                f'no available connections for deployment {deployment} and shard {shard_id}'
             )
             return None
 
@@ -561,7 +623,7 @@ class GrpcConnectionPool:
         self,
         deployment: str,
         address: str,
-        head: Optional[bool] = False,
+        head: bool = False,
         shard_id: Optional[int] = None,
     ):
         """
@@ -578,12 +640,13 @@ class GrpcConnectionPool:
             if shard_id is None:
                 shard_id = 0
             self._connections.add_replica(deployment, shard_id, address)
+        self._deployment_address_map[deployment] = address
 
     async def remove_connection(
         self,
         deployment: str,
         address: str,
-        head: Optional[bool] = False,
+        head: bool = False,
         shard_id: Optional[int] = None,
     ):
         """
@@ -614,6 +677,39 @@ class GrpcConnectionPool:
         """
         await self._connections.close()
 
+    def _handle_aiorpcerror(
+        self,
+        e: AioRpcError,
+        retry_i: int = 0,
+        request_id: str = '',
+        dest_addr: str = '',
+    ):
+        # connection failures and cancelled requests should be retried
+        # all other cases should not be retried and will be raised immediately
+        # connection failures have the code grpc.StatusCode.UNAVAILABLE
+        # cancelled requests have the code grpc.StatusCode.CANCELLED
+        # requests usually gets cancelled when the server shuts down
+        # retries for cancelled requests will hit another replica in K8s
+        if (
+            e.code() != grpc.StatusCode.UNAVAILABLE
+            and e.code() != grpc.StatusCode.CANCELLED
+        ):
+            raise
+        elif e.code() == grpc.StatusCode.UNAVAILABLE and retry_i >= 2:
+            self._logger.debug(f'GRPC call failed, retries exhausted')
+            from jina.excepts import InternalNetworkError
+
+            raise InternalNetworkError(
+                og_exception=e,
+                request_id=request_id,
+                dest_addr=dest_addr,
+                details=e.details(),
+            )
+        else:
+            self._logger.debug(
+                f'GRPC call failed with code {e.code()}, retry attempt {retry_i + 1}/3'
+            )
+
     def _send_requests(
         self,
         requests: List[Request],
@@ -623,9 +719,7 @@ class GrpcConnectionPool:
     ) -> asyncio.Task:
         # this wraps the awaitable object from grpc as a coroutine so it can be used as a task
         # the grpc call function is not a coroutine but some _AioCall
-        async def task_wrapper(
-            requests, connection: GrpcConnectionPool.ConnectionStubs, endpoint
-        ):
+        async def task_wrapper():
             metadata = (('endpoint', endpoint),) if endpoint else None
             for i in range(3):
                 try:
@@ -636,33 +730,48 @@ class GrpcConnectionPool:
                         timeout=timeout,
                     )
                 except AioRpcError as e:
-                    # connection failures and cancelled requests should be retried
-                    # all other cases should not be retried and will be raised immediately
-                    # connection failures have the code grpc.StatusCode.UNAVAILABLE
-                    # cancelled requests have the code grpc.StatusCode.CANCELLED
-                    # requests usually gets cancelled when the server shuts down
-                    # retries for cancelled requests will hit another replica in K8s
-                    if (
-                        e.code() != grpc.StatusCode.UNAVAILABLE
-                        and e.code() != grpc.StatusCode.CANCELLED
-                    ):
-                        raise
-                    elif e.code() == grpc.StatusCode.UNAVAILABLE and i == 2:
-                        self._logger.debug(f'GRPC call failed, retries exhausted')
-                        raise
-                    else:
-                        self._logger.debug(
-                            f'GRPC call failed with code {e.code()}, retry attempt {i+1}/3'
-                        )
+                    self._handle_aiorpcerror(
+                        e=e,
+                        retry_i=i,
+                        request_id=requests[0].request_id,
+                        dest_addr=connection.address,
+                    )
 
-        return asyncio.create_task(task_wrapper(requests, connection, endpoint))
+        return asyncio.create_task(task_wrapper())
+
+    def _send_discover_endpoint(
+        self,
+        connection: ConnectionStubs,
+        timeout: Optional[float] = None,
+    ) -> asyncio.Task:
+        # this wraps the awaitable object from grpc as a coroutine so it can be used as a task
+        # the grpc call function is not a coroutine but some _AioCall
+        async def task_wrapper():
+            for i in range(3):
+                try:
+                    return await connection.send_discover_endpoint(
+                        timeout=timeout,
+                    )
+                except AioRpcError as e:
+                    self._handle_aiorpcerror(
+                        e=e, retry_i=i, dest_addr=connection.address
+                    )
+                except AttributeError:
+                    # in gateway2gateway communication, gateway does not expose this endpoint. So just send empty list which corresponds to all endpoints valid
+                    from jina import __default_endpoint__
+
+                    ep = jina_pb2.EndpointsProto()
+                    ep.endpoints.extend([__default_endpoint__])
+                    return ep, None
+
+        return asyncio.create_task(task_wrapper())
 
     @staticmethod
     def get_grpc_channel(
         address: str,
         options: Optional[list] = None,
-        asyncio: Optional[bool] = False,
-        tls: Optional[bool] = False,
+        asyncio: bool = False,
+        tls: bool = False,
         root_certificates: Optional[str] = None,
     ) -> grpc.Channel:
         """
@@ -781,7 +890,7 @@ class GrpcConnectionPool:
         endpoint: Optional[str] = None,
     ) -> Request:
         """
-        Sends a request synchronically to the target via grpc
+        Sends a request synchronously to the target via grpc
 
         :param request: the request to send
         :param target: where to send the request to, like 127.0.0.1:8080
